@@ -39,7 +39,7 @@
 //! * [`Generic methods`](#generic-methods)
 //! * [`Methods with generic lifetimes`](#methods-with-generic-lifetimes)
 //! * [`Generic traits and structs`](#generic-traits-and-structs)
-//! * [`Associated types`](#associated-types-1)
+//! * [`Associated types`](#associated-types)
 //! * [`Multiple and inherited traits`](#multiple-and-inherited-traits)
 //! * [`External traits`](#external-traits)
 //! * [`Static methods`](#static-methods)
@@ -668,8 +668,8 @@
 //! }
 //! # fn main() {}
 //! ```
-//! For structs with more than one `impl` block, see [`mock!`]
-//! instead.
+//! For structs with more than one `impl` block or that have unsupported
+//! `#[derive(X)]` attributes, e.g. `Clone`, see [`mock!`] instead.
 //!
 //! ## Generic methods
 //!
@@ -963,6 +963,9 @@
 //! "mock_xxx", if "xxx" is the original module's name.  You can also use
 //! `#[double]` to selectively import the mock module.
 //!
+//! Be careful!  Module functions are static and so have the same caveats as
+//! [static methods](#static-methods) described above.
+//!
 //! ```
 //! # use mockall::*;
 //! # use mockall_double::*;
@@ -1289,6 +1292,23 @@ pub use mockall_derive::automock;
 /// }
 /// # fn main() {}
 /// ```
+/// Mocking an unsupported `#[derive(X)]` attribute, e.g. `Clone`, is
+/// similar.
+/// ```
+/// # use mockall_derive::mock;
+/// #[derive(Clone)]
+/// struct MyStruct;
+///
+/// mock!{
+///     pub MyStruct {
+///         fn bar(&self);
+///     }
+///     impl Clone for MyStruct {
+///         fn clone(&self) -> Self;
+///     }
+/// }
+/// # fn main() {}
+/// ```
 ///
 /// When mocking a generic struct's implementation of a generic trait, use the
 /// same namespace for their generic parameters.  For example, if you wanted to
@@ -1460,6 +1480,14 @@ impl From<RangeToInclusive<usize>> for TimesRange {
     }
 }
 
+#[derive(PartialEq)]
+#[doc(hidden)]
+pub enum ExpectedCalls {
+    Satisfied,
+    TooMany,
+    TooFew,
+}
+
 #[derive(Debug, Default)]
 #[doc(hidden)]
 pub struct Times{
@@ -1508,10 +1536,23 @@ impl Times {
         (self.range.0.end - self.range.0.start) == 1
     }
 
-    /// Has this expectation already been called the minimum required number of
-    /// times?
-    pub fn is_satisfied(&self) -> bool {
-        self.count.load(Ordering::Relaxed) >= self.range.0.start
+    /// Has this expectation already been called the expected number of times?
+    /// If not, was it too many or too few?
+    pub fn is_satisfied(&self) -> ExpectedCalls {
+        let satisfied_lower_bound = self.count.load(Ordering::Relaxed) >= self.range.0.start;
+        let satisfied_upper_bound = self.count.load(Ordering::Relaxed) < self.range.0.end;
+        if satisfied_lower_bound && satisfied_upper_bound {
+            ExpectedCalls::Satisfied
+        } else if satisfied_lower_bound {
+            ExpectedCalls::TooMany
+        } else {
+            ExpectedCalls::TooFew
+        }
+    }
+
+    /// The maximum number of times that this expectation must be called
+    pub fn maximum(&self) -> usize {
+        self.range.0.end - 1
     }
 
     /// The minimum number of times that this expectation must be called
@@ -1546,7 +1587,7 @@ pub struct Key(any::TypeId);
 
 #[doc(hidden)]
 impl Key {
-    pub fn new<T: 'static>() -> Self {
+    pub fn new<T: 'static + ?Sized>() -> Self {
         Key(any::TypeId::of::<T>())
     }
 }
